@@ -8,7 +8,7 @@ use strict;
 
 package Date::Set;
 
-use Set::Infinite;
+use Set::Infinite qw(trace_open trace_close);
 use vars qw(@ISA @EXPORT @EXPORT_OK  $AUTOLOAD $VERSION
   %FREQ %WEEKDAY %WHICH_OCCURRENCE
   $FUTURE $PAST $FOREVER $NEVER
@@ -21,7 +21,7 @@ use Carp;
 @ISA       = qw(Set::Infinite);
 @EXPORT    = qw();
 @EXPORT_OK = qw(type $inf inf);
-$VERSION = (qw'$Revision: 1.24 $')[1];
+$VERSION = (qw'$Revision: 1.25 $')[1];
 
 
 #----- initialize package globals
@@ -862,11 +862,12 @@ sub dtstart {    # start
     my ( $self, %parm ) = @_;
     unless (ref($self)) {
         # constructor
-        $self = $self->new( $parm{start}, $FUTURE );
+        $self = $self->new( $parm{start} )->offset( value => [ 0, $inf ] );   # $parm{start}, $FUTURE );
         $self->{dtstart} = $parm{start};
+        $self->trace( title => 'dtstart-new' );
         return $self;
     }
-    $self->_print( title => 'dtstart ' . join ( ':', %parm ) ) if $DEBUG;
+    $self->trace( title => 'dtstart ' . join ( ':', %parm ) );
     my $dt = $self->copy;
     $dt->{dtstart} = $parm{start};
     # print " dtstart $self->{dtstart}\n";
@@ -998,7 +999,7 @@ __internal_docs__
 
 sub rrule {
     my $self = shift;
-    carp ref $self . "->rrule deprecated in favor of recur_by_rule";
+    # carp ref $self . "->rrule deprecated in favor of recur_by_rule";
     return ( $self->recur_by_rule(@_) );
 }
 
@@ -1030,7 +1031,8 @@ sub recur_by_rule {
     my $self = shift;
 
     # $self = $self->numeric;
-    # carp "recur_by_rule $self";
+    $self->trace_open( title => "recur_by_rule" );
+    #trace_open();
 
     # TODO - put 'if' around the parsing so that 'backtrack' doesn't have
     # to redo it
@@ -1052,19 +1054,19 @@ sub recur_by_rule {
         }
         $last_p = $p;
     }
-    print "\nRECUR_BY_RULE ",join(":",@_),"\n" if $DEBUG;
+    $self->trace( title => "recur_by_rule" );   # ".join(":",@_) );
 
     my %parm = (
         # FREQ     => 'YEARLY',  # whatever
         INTERVAL => 1,
-        COUNT    => 1e10,    # any big number
+        COUNT    => $inf, 
         UNTIL    => undef,
         # WKST     => $WKST,
         RRULE    => undef,
         PERIOD   => undef, 
         DTSTART  => undef,
         INCLUDE_DTSTART => 1,  # *ALWAYS* include DTSTART in result set
-		EXCLUDE_DTSTART => 0,  # DON'T remove DTSTART from result set
+        EXCLUDE_DTSTART => 0,  # DON'T remove DTSTART from result set
         @parm,
     );
 
@@ -1097,9 +1099,12 @@ sub recur_by_rule {
     $parm{PERIOD} = $parm{PERIOD}->numeric if defined $parm{PERIOD};
 
 
-    $parm{DTSTART}   = $self->{dtstart} if defined $self->{dtstart};
-    $parm{DTSTART}   = $parm{PERIOD}->{dtstart} if ref($parm{PERIOD}) and defined $parm{PERIOD}->{dtstart} and not defined $parm{DTSTART};
-    $parm{DTSTART}   = $self->new( $parm{DTSTART} )->min if defined $parm{DTSTART};
+    $parm{DTSTART}   = $self->{dtstart} 
+        if defined $self->{dtstart} and not defined $parm{DTSTART};
+    $parm{DTSTART}   = $parm{PERIOD}->{dtstart} 
+        if ref($parm{PERIOD}) and defined $parm{PERIOD}->{dtstart} and not defined $parm{DTSTART};
+    $parm{DTSTART}   = $self->new( $parm{DTSTART} )->min 
+        if defined $parm{DTSTART};
     $parm{DTSTART_save} = $parm{DTSTART};   
 
     # print " dtstart is $parm{DTSTART} , was $self->{dtstart} \n";
@@ -1111,11 +1116,11 @@ sub recur_by_rule {
         $has{period} = 1;
         print " PERIOD: $parm{PERIOD} \n" if $DEBUG;
         # try to make $self smaller
+
+        # TODO: test for too_complex PERIOD? (we test $self later)
         $self = $self->intersection( $parm{PERIOD} );
 
         print "  self intersected with period\n" if $DEBUG;
-
-        # $self->{dtstart} = $parm{PERIOD}->{dtstart} if ref($parm{PERIOD}) and defined $parm{PERIOD}->{dtstart} and not defined $self->{dtstart};
     }
     else {
         $has{period}  = 0;
@@ -1133,11 +1138,11 @@ sub recur_by_rule {
             if ( defined $parm{DTSTART_save} ) {
                 # print "  ( $parm{DTSTART_save} < $self->min )  \n";
                 if ( ( $parm{DTSTART_save} < $self->min ) and 
-                     ($parm{COUNT} != 1e10) ) {
+                     ($parm{COUNT} < $inf) ) {
                     $self = $self->union( $parm{DTSTART_save}, $self->min );
                 }
                 if ( $parm{DTSTART_save} > $self->min ) {
-                    $self = $self->intersection( $parm{DTSTART_save}, inf );
+                    $self = $self->intersection( $parm{DTSTART_save}, $inf );
                 }
             }
 
@@ -1157,6 +1162,17 @@ sub recur_by_rule {
         $self->_print( title => 'UNTIL' ) if $DEBUG;
         $self = $self->intersection( $PAST, $parm{UNTIL} );
     }
+    elsif (exists $parm{COUNT}  &&
+           exists $parm{FREQ}   &&
+           ($parm{COUNT} != &inf) &&
+           ($self->min   != -&inf) ) {
+        # warn "we have COUNT=$parm{COUNT} INTERVAL=$parm{INTERVAL} and start=".$self->min." and FREQ=$parm{FREQ}";
+        my $count = $parm{COUNT} * $parm{INTERVAL};
+        my $try = $self->new( $self->min )->
+                offset( unit => $FREQ{$parm{FREQ}}, value => [0,$count] );
+        # warn "set = $try";
+        $self = $self->intersection($try);
+    }
 
     # this is the backtracking interface.
     # It allows the program to defer processing if it does not have enough
@@ -1166,20 +1182,63 @@ sub recur_by_rule {
         or ( $self->min and $self->min == -&inf )
         or ( $self->max and $self->max == &inf ) )
     {
+
         my $b = $self->new();
         $self->trace( title => "rrule:backtrack" );
         print " BACKTRACKING \n" if $DEBUG;
 
         # print " [rrule:backtrack] \n" if $DEBUG_BT;
-        $b->{too_complex} = 1;
-        $b->{parent}      = $self->copy;
-        $b->{method}      = 'recur_by_rule';
-        $b->{param}       = [%parm];
+        $b = $self->_function( 'recur_by_rule', %parm );
+        # $b->{too_complex} = 1;
+        # $b->{parent}      = $self;  #->copy;
+        # $b->{method}      = 'recur_by_rule';
+        # $b->{param}       = [%parm];
+
+        # TODO: set up {min} and {max}, if possible...
+
+        # try to make a subset suitable for first() iteration
+        # warn "trying 'first': ";
+        # TODO: delay this it until we are actually first()'ing
+        if (exists $parm{FREQ}   &&
+            ($self->min   != -&inf)
+           ) {
+            # warn "we have INTERVAL=$parm{INTERVAL} and start=".$self->min." and FREQ=$parm{FREQ}";
+            my $count = $parm{INTERVAL};  # if we had COUNT we weren't here
+            my $try = $self->new( $self->min )->
+                offset( unit => $FREQ{$parm{FREQ}}, mode => 'begin', value => [0,$count] );
+            $try->{list}[0]{open_end} = 1;  # TODO: this is not very "OO"
+            # warn "\n    set = $try";
+            my $subset = $b->intersection($try);
+            # warn "\n    subset = $subset";
+            my $size = 1 + $#{$subset->{list}};
+            if ( $size > 0 ) {
+                # warn "\n    self: $self \n    parm: ".join(' ', map { "$_=$parm{$_}" } keys %parm );
+                # move $self->min ahead
+                my $tail = $b->{parent}->complement($try)->_function( 'recur_by_rule', %parm );
+                my @first = $subset->first;
+                $tail = $tail->union($first[1]) if defined $first[1];
+                @{$b->{first}} = ($first[0], $tail);
+                # warn "    parent: ".$b->{parent};
+                # TODO: save this as first-cache
+                # my $tmp = $subset->union($b);
+                # trace_close( arg => $tmp );
+                # return $tmp;
+            }
+            # else {
+            #    # warn "failed: 'first' $size";
+            # }
+        }
+        else {
+            $b->{first} = [ $b->new(-$inf), $b->copy ];
+        }
+
+
+        $b->trace_close( arg => $b );
         return $b;
     }
 
     print "  self numeric\n" if $DEBUG;
-    $self = $self->numeric;
+    $self = $self->numeric;   # again?
     # $parm{PERIOD} = $parm{PERIOD}->numeric if $has{period};
     # $parm{DTSTART} = $parm{DTSTART}->numeric if defined $parm{DTSTART};
 
@@ -1188,32 +1247,10 @@ sub recur_by_rule {
 
     my $when = $parm{PERIOD};
 
-    # NOTE: is this redundant? maybe already had it checked above...
-    # DTSTART gives the default values for month, day, h, m, s
-    unless ( defined $parm{DTSTART} ) {
-        $parm{DTSTART} = $parm{PERIOD}->min;
-    }
-    else {
-        # make sure DTSTART is the right type
-        print "  make sure DTSTART is the right type\n" if $DEBUG;
-        $parm{DTSTART} = $self->new( $parm{DTSTART} )->min;
-
-        # apply DTSTART, just in case
-        $when = $when->intersection( $parm{DTSTART}, $FUTURE );
-    }
-
     # print " PARAMETERS: ", join(":", %parm), "\n";
 
     $when->_print( title => 'WHEN' ) if $DEBUG;
 
-#    # UNTIL and COUNT MUST NOT occur in the same 'recur'  (why?)
-#    if ( $parm{UNTIL} ) {
-#
-#        # UNTIL
-#        $when->_print( title => 'UNTIL' ) if $DEBUG;
-#        $when = $when->intersection( $PAST, $parm{UNTIL} );
-#    }
-
     $parm{WKST} = $self->{wkst} unless defined $parm{WKST};
 
     if ( $parm{FREQ} ) {
@@ -1224,7 +1261,7 @@ sub recur_by_rule {
 
         $when->_print( title => 'FREQ' ) if $DEBUG;
 
-        if ( $self->max == &inf ) {
+        if ( $self->max == $inf ) {
 
             # TODO
             # that's real hard to quantize -- try to fix it
@@ -1270,13 +1307,14 @@ sub recur_by_rule {
         if ($parm{INTERVAL} > 1) {
             $freq = $freq
               # -- INTERVAL works here:
-              ->select( freq => $parm{INTERVAL}, count => 999999, strict => 0 )
-              ->_print(
+              ->select( freq => $parm{INTERVAL}, count => $inf, strict => 0 );
+            # carp "INTERVAL $parm{INTERVAL}";
+            $freq->_print(
                 title => 'FREQ('
                   . $parm{FREQ}
                   . ')+INTERVAL('
                   . $parm{INTERVAL} . ')'
-              )->compact;
+              )->compact if $DEBUG;
         }
 
         # -- BYSETPOS special handling -- 
@@ -1303,12 +1341,16 @@ sub recur_by_rule {
         $rrule = $when->intersection( $freq
               ->_apply_DTSTART( \%parm , \%has )
 
+              # offset to begin before intersection, because
+              # intersection would 'join' everything
+              # ->offset(mode=>'begin', value=>[0,0])
+ 
               # remove anything out of range before counting!
-              ->intersection( $parm{PERIOD} )
+              ->intersection( $parm{PERIOD} )->no_cleanup
 
               # -- COUNT works here:
               ->select( freq => 1, count => $parm{COUNT}, strict => 0 )
-              ->_print( title => 'COUNT(' . $parm{COUNT} . ')' )
+              # ->_print( title => 'COUNT(' . $parm{COUNT} . ')' )
 
               # ->duration( unit => 'seconds', duration => 0 ) 
               ->offset(mode=>'begin', value=>[0,0])
@@ -1341,10 +1383,15 @@ sub recur_by_rule {
     # carp "        as " . $rrule->fixtype ;
     # carp "       and " . $self->fixtype ;
 
+    my $tmp;
     if ( $has{period} ) {
-        return $self->union($rrule)->fixtype;
+        $tmp = $self->union($rrule)->fixtype;
     }
-    return $rrule->fixtype;
+    else {
+        $tmp = $rrule->fixtype;
+    }
+    trace_close;
+    return $tmp;
 }
 
 
@@ -1406,11 +1453,8 @@ sub _apply_DTSTART {
 
 # INTERNAL sub to calculate BYxxx
 #   input: $when, %parm
-
 #   output: $when (filtered)
 
-# TODO: This code needs to be refactored into ~ <50 line chunks and tested
-
 sub _rrule_by {
 
     print " [EVALUATING RRULE ",join(':',@_)," ]\n" if $DEBUG;
@@ -1515,6 +1559,7 @@ sub _rrule_by {
             # reuse "base" since we already know it from FREQ
             # -- iterate through parameters
             $indexed = $NEVER;
+            # $DEBUG = 1;
             foreach (@indexed_byday) {
                 # parse parameters
                 my ( $index, $day ) = /([\-\+]?\d+)(\w\w)/;
@@ -1522,7 +1567,8 @@ sub _rrule_by {
                 $index-- if $index > 0;   # perl index starts in 0 instead of 1
                 $day = $WEEKDAY{$day};
                 # print " [Indexed BYDAY: $index $day ]\n" if $DEBUG;
-                my $weekday = $base->offset( 
+                ## my $weekday = $base->offset( 
+                my $weekday = $BYDAY->offset(
                     mode => 'offset', 
                     unit => 'weekdays', 
                     value => [ $day, $day ] );
@@ -1548,6 +1594,7 @@ sub _rrule_by {
 
         $has{months} = 1;
         $has{days}   = 1;
+        # $DEBUG = 0;
     }    # end: BYDAY
     # }}}
 
@@ -1565,6 +1612,7 @@ sub _rrule_by {
             $when = $when->intersection(
                 $when->quantize(
                     unit => $prev_unit,
+                    strict => 0,
                     fixtype => 0 )
                  ->offset(
                     mode   => 'circle',
@@ -1583,13 +1631,16 @@ sub _rrule_by {
     if ( exists $parm{BYSETPOS} ) {
         my @by = @{ $parm{BYSETPOS} };
         foreach (@by) { $_-- if $_ > 0 }    # BY starts in 1; perl starts in 0
-        $when = $when->intersection( 
-           $when->compact
-           # ->_print(title=>'bysetpos1')
-           ->select( by => [@by] )
-           # ->_print(title=>'bysetpos2')
-        )->no_cleanup;
+        # $when = $when->intersection( 
+        #   $when->compact
+        #   # ->_print(title=>'bysetpos1')
+        #   ->select( by => [@by] )
+        #   # ->_print(title=>'bysetpos2')
+        # )->no_cleanup;
         $when->_print( title => 'BYSETPOS' ) if $DEBUG;
+
+        $when = $when->select( by => [@by] );
+        # carp " When: $when ";
     }
     # }}} 
 
@@ -1648,7 +1699,7 @@ sub exclude_by_rule {
 
         # print " [exclude_by_rule:backtrack] \n" if $DEBUG_BT;
         $b->{too_complex} = 1;
-        $b->{parent}      = $self->copy;
+        $b->{parent}      = $self;  # ->copy;
         $b->{method}      = 'exclude_by_rule';
         $b->{param}       = \@_;
         return $b;
@@ -1826,6 +1877,8 @@ See also previous note on 'weekyear' in 'About ISO 8601 week'.
 
 =cut
 
+sub DESTROY {}
+
 # try to make up for unwritten methods
 # currently defined are:
 #   next_XXX  (next_day, next_month ...), this_XXX,  prev_XXX
