@@ -44,7 +44,8 @@ use overload
         $_[2] ? ($_[1]  -  $_[0]{epoch}) : ($_[0]{epoch}  -  $_[1]) },
     '+' =>   sub { $_[0]->{epoch} + $_[1] },
     qw("" as_string),
-    # fallback => 1;
+    fallback => 1;   # we need this for the modulo "%" operation in
+                     # quantize() initialization to work
 ;
 
 %NEW_CACHE = ();
@@ -77,7 +78,15 @@ sub new {
 
     # figure out what kind of parameter we were given and 
     # get it in a standard format, an iCalendar string
-    return $string if ref($string);
+    if ( ref($string) ) {
+        if ( UNIVERSAL::isa( $string, 'Date::ICal' )) {
+            $self = bless {}, __PACKAGE__;
+            $self->{ical}  = $string;
+            $self->{epoch} = $string->epoch;
+            return $NEW_CACHE{$string} = $self;  # cache object
+        }
+        return $string;
+    }
     return $NEW_CACHE{$string} if exists $NEW_CACHE{$string};
 
     # print " [ical:new:", join(';', @_) , "] ";
